@@ -3,7 +3,7 @@
 /*      asmail - AfterStep Mail                                              */
 /*                                                                           */
 /*      By Per Liden                                                         */
-/*      Last changed Feb 17. 2000 by Martin Domig                            */
+/*      Last changed Apr 13. 2000 by Martin Domig                            */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -20,6 +20,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <sys/types.h>
+#include <dirent.h>
 #ifdef __QNX__
 #include <X11/IntrinsicP.h> /* for XtResizeWidget() */
 #include <unix.h> /* for gethostname() */
@@ -73,13 +75,22 @@ Pixel GetColor(char *name);
 void AddXpmToAnim(XpmIcon * Icon, XpmIcon * List, int ListNo);
 void Quit(Widget w, XEvent *event, String *params, Cardinal *num_params);
 
+int IsMaildir(char *string);
 int IsPopBox(char *string);
 int TotalNewMail(void);
 void shorten(char *line);
 int TotalMail(void);
 
 /* Global stuff ************************************************************* */
-#define FILENAME		".asmailrc"
+
+/* overridable from the Makefile */
+#ifndef RCFILE
+ #define RCFILE                 ".asmailrc"
+#endif
+#ifndef DEBUGLEVEL
+ #define DEBUGLEVEL             0
+#endif
+
 #define DEFINTERVAL		5	/* Default interval 5 sec */
 #define DEFANIMATIONSPEED	0	/* Default animationspeed is none */
 #define LEFTBUTTON		1	/* Left mousebutton id */
@@ -166,7 +177,7 @@ void MakeSettings(int argc, char *argv[])
     if (argc == 1) {
 	strcpy(CfgFile, (char *) getenv("HOME"));
         strcat(CfgFile, "/");
-	strcat(CfgFile, FILENAME);
+	strcat(CfgFile, RCFILE);
     } else if (argc == 2) {
 	/* Check if option */
 	if (argv[1][0] == '-')
@@ -183,7 +194,7 @@ void MakeSettings(int argc, char *argv[])
 	ParseCfgFile(File);
 	fclose(File);
     } else
-	    fprintf(stderr, "asmail: '%s' not found in ~/, using default settings\n", FILENAME);
+	    fprintf(stderr, "asmail: '%s' not found in ~/, using default settings\n", RCFILE);
 
     /* Check interval */
     if (!Interval) {
@@ -674,6 +685,9 @@ void RedrawWindow(void)
 			gcv.foreground = NumOfMsgColor;
 			XChangeGC(Disp, WinGC, GCForeground, &gcv);
 			NumOfMsg.nchars = strlen(NumOfMsg.chars);
+#if ( DEBUGLEVEL >= 2 )
+			fprintf( stderr, "NumOfMsg.chars: %s\n", NumOfMsg.chars);
+#endif
 			XDrawText(Disp, XtWindow(MainWin), WinGC, x + NumOfMsgX, y + NumOfMsgY,
 					&NumOfMsg, 1);
 /*
@@ -926,8 +940,9 @@ int IsNewMail(char *Mbox, int index)
     char Buffer[30] = "";
     int NewMailn = 0;
     
-    if (IsPopBox(Mbox))
-        return NewMailCounter[index];
+
+    if( IsMaildir(Mbox) || IsPopBox(Mbox) )
+      return NewMailCounter[index];
     
     MailFile = fopen(Mbox, "r");
     MailCounter[index] = 0;
@@ -960,43 +975,140 @@ int IsPopBox(char *string)
         return 0;
 }
 /****************************************************************************/
+int IsMaildir(char *string)
+{
+  int n;
+
+  n = strlen(string);
+  return (string[n-1] == '/')? n:0;
+}
+/****************************************************************************/
 int IsMail(char *Mbox, int index)
 {
     FILE *MailFile = 0;
-    int Ret = 0;
-    char *Exec;
-    int ReturnVal, NewMailn, OldMails;
+    int Ret;
+    int namelen;
 
-    if (IsPopBox(Mbox)) {
-        Exec = (char *)malloc(strlen(Mbox));
-        strncpy(Exec, Mbox, strlen(Mbox)+1);
-        
-        if (Exec == NULL)
-            return 0;
-        ReturnVal = pop(Exec, &NewMailn, &OldMails);
-        free(Exec);
-        
-        if (ReturnVal >= 0) {
-            MailCounter[index] = NewMailn;
-            NewMailCounter[index] = NewMailn - OldMails;
-            return ReturnVal;
-        } else
-            return 0;
-    }
-        
-    MailFile = fopen(Mbox, "r");
-    if (MailFile == 0)
-	   return 0;
-    else {
-	   fgetc(MailFile);
-       if (feof(MailFile))
-           Ret = 0;
-       else
-           Ret = 1;
+    if( namelen = IsMaildir(Mbox) ) {
+      
+      char *d, dirstr[3][5] = { "new/", "cur/", "tmp/" };
+      DIR *MailDir;
+      struct dirent *DirEntry;
+      int n;
 
-       fclose(MailFile);
+      d = malloc( namelen + 5 );
+
+      strcpy( d, Mbox );
+
+      /*
+	Ret: 1 no mail, 2 some mail
+	if negative, exit while loop 
+	0: some error
+      */
+      Ret = 1;
+      NewMailCounter[index] = 0;
+      MailCounter[index] = 0;
+
+      /* go through the three subdirs of a maildir */
+      for( n=0; (n<3) && (Ret>0); ++n ) {
+	strcpy( &d[namelen], dirstr[n] );
+#if ( DEBUGLEVEL >= 2 )
+	fprintf( stderr, "Mode: %d, Directory: %s\n", NumOfMsgMode, d );
+#endif
+	MailDir = opendir( d );
+
+	/* if any of the dirs fails to open, fail */
+	if (MailDir == 0) Ret = 0;
+	else {
+	  /* go through the directory */
+	  while( (Ret>0) && (DirEntry=readdir(MailDir)) ) {
+	    /* if there is any normal file */
+#if ( DEBUGLEVEL >= 3 )
+	    fprintf(stderr, "  DirEntry->d_name: %s\n", DirEntry->d_name);
+#endif
+	    if( DirEntry->d_name[0] != '.' ) {
+	      /* i want to go through the whole dir only
+		 for some NumOfMsgModes, otherwise
+		 stop as soon as i find anything */
+	      if( n == 0) {
+		if ( NumOfMsgMode > 0 ) {
+		  ++NewMailCounter[index];
+		  ++MailCounter[index];
+		  Ret = 2;
+		} else {
+		  NewMailCounter[index] = 1;
+		  Ret = -2;
+		}
+	      }
+	      else if( (NumOfMsgMode==1) || (NumOfMsgMode==3) ) {
+		++MailCounter[index];
+		Ret = 2;
+	      }
+	      else Ret = -2;
+	    } /* endif normal file in d */
+	  } /* wend dir traversal */
+
+	  closedir(MailDir);
+        } /* endif MailDir */
+	  
+      } /* endfor */
+
+#if ( DEBUGLEVEL >= 1 )
+      if( !MailDir ) perror( "opendir() returned NULL - fd leak?" ); 
+#endif
+
+#if ( DEBUGLEVEL >= 2 )
+      fprintf( stderr, "  Ret: %d, Mail: %d, New Mail: %d\n",\
+	       Ret, MailCounter[index], NewMailCounter[index] );
+      fprintf( stderr, "====\n" );
+#endif
+
+      /* new semantics: 1 mail, 0 no mail or error */
+      if( (Ret==2) || (Ret==-2) ) Ret = 1;
+      else Ret = 0;
+      
+      free( d );
+      free( DirEntry );
+    }      
+
+    else if( IsPopBox(Mbox) ) {
+
+      int NewMailn, OldMails;
+      char *Exec;
+
+      Exec = (char *)malloc(strlen(Mbox));
+      if (Exec == NULL) return 0;                   /* EXIT POINT */
+
+      strncpy(Exec, Mbox, strlen(Mbox)+1);
+        
+      Ret = pop(Exec, &NewMailn, &OldMails);
+      free(Exec);
+        
+        
+      if (Ret >= 0) {
+        MailCounter[index] = OldMails;
+        NewMailCounter[index] = NewMailn;
+        return Ret;
+      } else
+        return 0;
     }
-    return Ret;
+
+    else {     /* mbox */
+      MailFile = fopen(Mbox, "r");
+      if (MailFile == 0)
+	return 0;                                   /* EXIT POINT */
+      else {
+	fgetc(MailFile);
+	if (feof(MailFile))
+	  Ret = 0;
+	else
+	  Ret = 1;
+
+	fclose(MailFile);
+      }
+
+    }
+    return Ret;                                   /* EXIT POINT */
 }
 
 /****************************************************************************/
