@@ -34,7 +34,7 @@
     ISBN 0-13-234709-1
     
    The original source has been modified by
-   MoGul <kaliph@gmx.net>			*/
+   Martin Domig <martin.domig@gmx.net>			*/
 
 
 SOCKET *Sopen(void) {
