@@ -16,6 +16,10 @@
 #include <netinet/in.h>
 #include <sys/types.h>
 
+#ifdef __QNX__
+#define O_NDELAY O_NONBLOCK
+#endif
+
 #define SOCKET_DELAY 0
 #define SOCKET_NDELAY 1
 
@@ -30,10 +34,10 @@ typedef struct {
    int sd;					/* Socket descriptor						*/
 } SOCKET;
 
-SOCKET *sopen(void);
-int sclose(SOCKET *);
-int sserver(SOCKET *, int, int);
-int sclient(SOCKET *, char *, int);
-size_t sread(int sd, char *string, int n, int timeout);
-size_t swrite(int sd, char *string);
+SOCKET *Sopen(void);
+int Sclose(SOCKET *);
+int Sserver(SOCKET *, int, int);
+int Sclient(SOCKET *, char *, int);
+size_t Sread(int sd, char *string, int n, int timeout);
+size_t Swrite(int sd, char *string);
 #include "socklib.c"
